@@ -17,12 +17,10 @@
 import argparse
 import sys
 import logging
-import signal
 from subiquitycore.log import setup_logger
 from subiquitycore import __version__ as VERSION
 from console_conf.core import ConsoleConf
 from subiquitycore.core import ApplicationError
-from subiquitycore.ui.frame import SubiquityUI
 from subiquitycore.utils import environment_check
 
 
@@ -83,19 +81,14 @@ def main():
     logger.info("Starting console-conf v{}".format(VERSION))
     logger.info("Arguments passed: {}".format(sys.argv))
 
-    signal.signal(signal.SIGINT, signal.SIG_IGN)
-    signal.signal(signal.SIGQUIT, signal.SIG_IGN)
-
     env_ok = environment_check(ENVIRONMENT)
     if env_ok is False and not opts.dry_run:
         print('Failed environment check.  '
               'Check {} for errors.'.format(LOGFILE))
         return 1
 
-    ui = SubiquityUI()
-
     try:
-        interface = ConsoleConf(ui, opts)
+        interface = ConsoleConf(opts)
     except ApplicationError as e:
         logger.exception('Failed to load ConsoleConf interface')
         print(e)
