@@ -15,7 +15,11 @@
 
 """ console-conf controllers """
 
-from .welcome import WelcomeController  # NOQA
-from subiquitycore.controllers.network import NetworkController  # NOQA
-from subiquitycore.controllers.login import LoginController  # NOQA
-from .identity import IdentityController  # NOQA
+from .identity import IdentityController
+from subiquitycore.controllers.network import NetworkController
+from .welcome import WelcomeController
+__all__ = [
+    'IdentityController',
+    'NetworkController',
+    'WelcomeController',
+]
