@@ -18,13 +18,11 @@ import argparse
 import logging
 import os
 import fcntl
-import signal
 import sys
 
 from subiquitycore.log import setup_logger
 from subiquitycore import __version__ as VERSION
 from subiquitycore.core import ApplicationError
-from subiquitycore.ui.frame import SubiquityUI
 from subiquitycore.utils import environment_check
 
 from subiquity.core import Subiquity
@@ -126,9 +124,6 @@ def main():
     logging.getLogger('curtin').addHandler(handler)
     logging.getLogger('block-discover').addHandler(handler)
 
-    signal.signal(signal.SIGINT, signal.SIG_IGN)
-    signal.signal(signal.SIGQUIT, signal.SIG_IGN)
-
     env_ok = environment_check(ENVIRONMENT)
     if env_ok is False and not opts.dry_run:
         print('Failed environment check.  '
@@ -149,10 +144,8 @@ def main():
             opts.answers.close()
             opts.answers = None
 
-    ui = SubiquityUI()
-
     try:
-        subiquity_interface = Subiquity(ui, opts, block_log_dir)
+        subiquity_interface = Subiquity(opts, block_log_dir)
     except ApplicationError as e:
         logger.exception('Failed to load Subiquity interface')
         print(e)
