@@ -56,18 +56,15 @@ class ProbeState(enum.IntEnum):
 
 class FilesystemController(BaseController):
 
-    def __init__(self, common):
-        super().__init__(common)
-        self.block_log_dir = common.get('block_log_dir')
-        self.model = self.base_model.filesystem
+    def __init__(self, app):
+        super().__init__(app)
+        self.model = app.base_model.filesystem
         if self.opts.dry_run and self.opts.bootloader:
             name = self.opts.bootloader.upper()
             self.model.bootloader = getattr(Bootloader, name)
-        self.answers = self.all_answers.get("Filesystem", {})
         self.answers.setdefault('guided', False)
         self.answers.setdefault('guided-index', 0)
         self.answers.setdefault('manual', [])
-        self.showing = False
         self._probe_state = ProbeState.NOT_STARTED
 
     def start(self):
@@ -78,7 +75,7 @@ class FilesystemController(BaseController):
             5.0, lambda loop, ud: self._check_probe_timeout())
 
     def _bg_probe(self, probe_types=None):
-        return self.prober.get_storage(probe_types=probe_types)
+        return self.app.prober.get_storage(probe_types=probe_types)
 
     def _probed(self, fut, restricted=False):
         if not restricted and self._probe_state != ProbeState.PROBING:
@@ -91,7 +88,7 @@ class FilesystemController(BaseController):
                 fname = 'probe-data-restricted.json'
             else:
                 fname = 'probe-data.json'
-            with open(os.path.join(self.block_log_dir, fname), 'w') as fp:
+            with open(os.path.join(self.app.block_log_dir, fname), 'w') as fp:
                 json.dump(storage, fp, indent=4)
             self.model.load_probe_data(storage)
         except Exception:
@@ -127,8 +124,7 @@ class FilesystemController(BaseController):
             lambda fut: self._probed(fut, True),
             )
 
-    def default(self):
-        self.showing = True
+    def start_ui(self):
         if self._probe_state in [ProbeState.PROBING,
                                  ProbeState.REPROBING]:
             self.ui.set_body(SlowProbing(self))
@@ -194,11 +190,11 @@ class FilesystemController(BaseController):
         if 'obj' in action:
             obj = self._action_get(action['obj'])
             meth = getattr(
-                self.ui.frame.body.avail_list,
+                self.ui.body.avail_list,
                 "_{}_{}".format(obj.type, action['action']))
             meth(obj)
             yield
-            body = self.ui.frame.body._w
+            body = self.ui.body._w
             if not isinstance(body, StretchyOverlay):
                 return
             if isinstance(body.stretchy, ConfirmDeleteStretchy):
@@ -210,25 +206,25 @@ class FilesystemController(BaseController):
                     action['data'],
                     action.get("submit", True))
         elif action['action'] == 'create-raid':
-            self.ui.frame.body.create_raid()
+            self.ui.body.create_raid()
             yield
-            body = self.ui.frame.body._w
+            body = self.ui.body._w
             yield from self._enter_form_data(
                 body.stretchy.form,
                 action['data'],
                 action.get("submit", True),
                 clean_suffix='raid')
         elif action['action'] == 'create-vg':
-            self.ui.frame.body.create_vg()
+            self.ui.body.create_vg()
             yield
-            body = self.ui.frame.body._w
+            body = self.ui.body._w
             yield from self._enter_form_data(
                 body.stretchy.form,
                 action['data'],
                 action.get("submit", True),
                 clean_suffix='vg')
         elif action['action'] == 'done':
-            if not self.ui.frame.body.done.enabled:
+            if not self.ui.body.done.enabled:
                 raise Exception("answers did not provide complete fs config")
             self.finish()
         else:
@@ -256,11 +252,9 @@ class FilesystemController(BaseController):
         self.manual()
 
     def cancel(self):
-        self.showing = False
         self.signal.emit_signal('prev-screen')
 
     def finish(self):
-        self.showing = False
         log.debug("FilesystemController.finish next-screen")
         # start curtin install in background
         self.signal.emit_signal('installprogress:filesystem-config-done')
