@@ -24,12 +24,11 @@ log = logging.getLogger('subiquity.controllers.identity')
 
 class IdentityController(BaseController):
 
-    def __init__(self, common):
-        super().__init__(common)
-        self.model = self.base_model.identity
-        self.answers = self.all_answers.get('Identity', {})
+    def __init__(self, app):
+        super().__init__(app)
+        self.model = app.base_model.identity
 
-    def default(self):
+    def start_ui(self):
         self.ui.set_body(IdentityView(self.model, self))
         if all(elem in self.answers for elem in
                ['realname', 'username', 'password', 'hostname']):
