@@ -22,6 +22,7 @@ import signal
 import subprocess
 import sys
 import platform
+import tempfile
 import time
 import traceback
 
@@ -205,12 +206,11 @@ class InstallProgressController(BaseController):
         ('installprogress:snap-config-done',       'snap_config_done'),
     ]
 
-    def __init__(self, common):
-        super().__init__(common)
-        self.answers = self.all_answers.get('InstallProgress', {})
+    def __init__(self, app):
+        super().__init__(app)
+        self.model = app.base_model
         self.answers.setdefault('reboot', False)
         self.progress_view = None
-        self.progress_view_showing = False
         self.install_state = InstallState.NOT_STARTED
         self.journal_listener_handle = None
         self._postinstall_prerequisites = {
@@ -225,7 +225,7 @@ class InstallProgressController(BaseController):
         self.sm = None
 
     def tpath(self, *path):
-        return os.path.join(self.base_model.target, *path)
+        return os.path.join(self.model.target, *path)
 
     def filesystem_config_done(self):
         self.curtin_start_install()
@@ -335,7 +335,7 @@ class InstallProgressController(BaseController):
 
         ident = self._event_syslog_identifier
         self._write_config(config_location,
-                           self.base_model.render(syslog_identifier=ident))
+                           self.model.render(syslog_identifier=ident))
 
         return curtin_cmd
 
@@ -372,7 +372,7 @@ class InstallProgressController(BaseController):
         self.install_state = InstallState.DONE
         log.debug('After curtin install OK')
         self.ui.progress_current += 1
-        if not self.progress_view_showing:
+        if not self.showing:
             self.ui.set_footer(_("Install complete"))
         else:
             # Re-set footer so progress bar updates.
@@ -383,13 +383,13 @@ class InstallProgressController(BaseController):
         pass
 
     def start_postinstall_configuration(self):
-        has_network = self.base_model.network.has_network
+        has_network = self.model.network.has_network
 
         def filter_task(func):
             if func._extra.get('net_only') and not has_network:
                 return False
             if func._name == 'install_openssh' \
-               and not self.base_model.ssh.install_server:
+               and not self.model.ssh.install_server:
                 return False
             return True
 
@@ -411,12 +411,12 @@ class InstallProgressController(BaseController):
 
     @task(label="configuring cloud-init")
     def _bg_configure_cloud_init(self):
-        self.base_model.configure_cloud_init()
+        self.model.configure_cloud_init()
 
     @task(label="installing openssh")
     def _bg_install_openssh(self):
         if self.opts.dry_run:
-            cmd = ["sleep", str(2/self.scale_factor)]
+            cmd = ["sleep", str(2/self.app.scale_factor)]
         else:
             cmd = [
                 sys.executable, "-m", "curtin", "system-install", "-t",
@@ -428,12 +428,12 @@ class InstallProgressController(BaseController):
     @task(label="restoring apt configuration")
     def _bg_restore_apt_config(self):
         if self.opts.dry_run:
-            cmds = [["sleep", str(1/self.scale_factor)]]
+            cmds = [["sleep", str(1/self.app.scale_factor)]]
         else:
             cmds = [
                 ["umount", self.tpath('etc/apt')],
                 ]
-            if self.base_model.network.has_network:
+            if self.model.network.has_network:
                 cmds.append([
                     sys.executable, "-m", "curtin", "in-target", "-t",
                     "/target", "--", "apt-get", "update",
@@ -459,14 +459,24 @@ class InstallProgressController(BaseController):
           transitions={'reboot': 'abort_uu'},
           net_only=True)
     def _bg_run_uu(self):
+        target_tmp = os.path.join(self.model.target, "tmp")
+        os.makedirs(target_tmp, exist_ok=True)
+        apt_conf = tempfile.NamedTemporaryFile(
+            dir=target_tmp, delete=False, mode='w')
+        apt_conf.write(uu_apt_conf)
+        apt_conf.close()
+        env = os.environ.copy()
+        env["APT_CONFIG"] = apt_conf.name[len(self.model.target):]
         if self.opts.dry_run:
-            self.uu = utils.start_command(["sleep", str(10/self.scale_factor)])
+            self.uu = utils.start_command([
+                "sleep", str(10/self.app.scale_factor)])
             self.uu.wait()
         else:
             self._bg_run_command_logged([
                 sys.executable, "-m", "curtin", "in-target", "-t", "/target",
                 "--", "unattended-upgrades", "-v",
-            ], check=True)
+            ], env=env, check=True)
+        os.remove(apt_conf.name)
 
     @task(transitions={'success': 'copy_logs_to_target'}, net_only=True)
     def uu_done(self):
@@ -553,8 +563,7 @@ class InstallProgressController(BaseController):
             utils.disable_subiquity()
         self.signal.emit_signal('quit')
 
-    def default(self):
-        self.progress_view_showing = True
+    def start_ui(self):
         if self.install_state == InstallState.RUNNING:
             self.progress_view.title = _("Installing system")
             footer = _("Thank you for using Ubuntu!")
@@ -567,3 +576,11 @@ class InstallProgressController(BaseController):
             footer = _('Please report this error in Launchpad')
         self.ui.set_body(self.progress_view)
         self.ui.set_footer(footer)
+
+
+uu_apt_conf = """\
+# Config for the unattended-upgrades run to avoid failing on battery power or
+# a metered connection.
+Unattended-Upgrade::OnlyOnACPower "false";
+Unattended-Upgrade::Skip-Updates-On-Metered-Connections "true";
+"""
