@@ -26,10 +26,9 @@ log = logging.getLogger('subiquity.controllers.welcome')
 
 class WelcomeController(BaseController):
 
-    def __init__(self, common):
-        super().__init__(common)
-        self.model = self.base_model.locale
-        self.answers = self.all_answers.get("Welcome", {})
+    def __init__(self, app):
+        super().__init__(app)
+        self.model = app.base_model.locale
         log.debug("Welcome: answers=%s", self.answers)
 
     def start(self):
@@ -40,7 +39,7 @@ class WelcomeController(BaseController):
             if code == lang:
                 self.model.switch_language(code)
 
-    def default(self):
+    def start_ui(self):
         view = WelcomeView(self.model, self)
         self.ui.set_body(view)
         if 'lang' in self.answers:
