@@ -629,9 +629,8 @@ class ZdevInfo:
 
 class ZdevController(BaseController):
 
-    def __init__(self, common):
-        super().__init__(common)
-        self.answers = self.all_answers.get('Zdev', {})
+    def __init__(self, app):
+        super().__init__(app)
         if self.opts.dry_run:
             if platform.machine() == 's390x':
                 zdevinfos = self.lszdev()
@@ -641,7 +640,7 @@ class ZdevController(BaseController):
                 zdevinfos = [ZdevInfo.from_row(row) for row in devices]
             self.zdevinfos = OrderedDict([(i.id, i) for i in zdevinfos])
 
-    def default(self):
+    def start_ui(self):
         if 'accept-default' in self.answers:
             self.done()
         self.ui.set_body(ZdevView(self))
