@@ -24,6 +24,7 @@ from subiquity.snapd import (
     FakeSnapdConnection,
     SnapdConnection,
     )
+from subiquity.ui.frame import SubiquityUI
 
 
 log = logging.getLogger('subiquity.core')
@@ -37,11 +38,14 @@ class Subiquity(Application):
 
     project = "subiquity"
 
-    def make_model(self, common):
+    def make_model(self):
         root = '/'
-        if common['opts'].dry_run:
+        if self.opts.dry_run:
             root = os.path.abspath('.subiquity')
-        return SubiquityModel(root, common['opts'].sources)
+        return SubiquityModel(root, self.opts.sources)
+
+    def make_ui(self):
+        return SubiquityUI(self)
 
     controllers = [
             "Welcome",
@@ -59,13 +63,13 @@ class Subiquity(Application):
             "InstallProgress",
     ]
 
-    def __init__(self, ui, opts, block_log_dir):
+    def __init__(self, opts, block_log_dir):
         if not opts.bootloader == 'none' and platform.machine() != 's390x':
             self.controllers.remove("Zdev")
 
-        super().__init__(ui, opts)
-        self.common['ui'].progress_completion += 1
-        self.common['block_log_dir'] = block_log_dir
+        super().__init__(opts)
+        self.ui.progress_completion += 1
+        self.block_log_dir = block_log_dir
         if opts.snaps_from_examples:
             connection = FakeSnapdConnection(
                 os.path.join(
@@ -74,22 +78,19 @@ class Subiquity(Application):
                     "examples", "snaps"))
         else:
             connection = SnapdConnection(self.root, self.snapd_socket_path)
-        self.common['snapd_connection'] = connection
-        signal = self.common['signal']
-        signal.connect_signals([
+        self.snapd_connection = connection
+        self.signal.connect_signals([
             ('network-proxy-set', self._proxy_set),
             ('network-change', self._network_change),
             ])
 
     def _network_change(self):
-        self.common['signal'].emit_signal('snapd-network-change')
+        self.signal.emit_signal('snapd-network-change')
 
     def _proxy_set(self):
-        proxy_model = self.common['base_model'].proxy
-        signal = self.common['signal']
-        conn = self.common['snapd_connection']
         self.run_in_bg(
-            lambda: conn.configure_proxy(proxy_model),
+            lambda: self.snapd_connection.configure_proxy(
+                self.base_model.proxy),
             lambda fut: (
-                fut.result(), signal.emit_signal('snapd-network-change')),
+                fut.result(), self.signal.emit_signal('snapd-network-change')),
             )
