@@ -13,17 +13,30 @@
 # You should have received a copy of the GNU Affero General Public License
 # along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
-from .filesystem import (FilesystemView,  # NOQA
-                         AddPartitionView,
-                         AddFormatView,
-                         DiskPartitionView,
-                         DiskInfoView)
-from .bcache import BcacheView  # NOQA
-from .raid import RaidView  # NOQA
-from .ceph import CephDiskView  # NOQA
-from .iscsi import IscsiDiskView  # NOQA
-from .lvm import LVMVolumeGroupView  # NOQA
-from .identity import IdentityView  # NOQA
-from .installpath import InstallpathView  # NOQA
-from .installprogress import ProgressView  # NOQA
+from .filesystem import (
+    FilesystemView,
+    GuidedDiskSelectionView,
+    GuidedFilesystemView,
+    )
+from .bcache import BcacheView
+from .ceph import CephDiskView
+from .iscsi import IscsiDiskView
+from .identity import IdentityView
+from .installprogress import ProgressView
+from .keyboard import KeyboardView
 from .welcome import WelcomeView
+from .zdev import ZdevView
+__all__ = [
+    'BcacheView',
+    'CephDiskView',
+    'FilesystemView',
+    'GuidedDiskSelectionView',
+    'GuidedFilesystemView',
+    'IdentityView',
+    'IscsiDiskView',
+    'KeyboardView',
+    'MAASView',
+    'ProgressView',
+    'WelcomeView',
+    'ZdevView',
+]
