@@ -22,7 +22,7 @@ from subiquitycore.ui.container import ListBox, Pile
 from subiquitycore.ui.interactive import Selector
 from subiquitycore.ui.utils import Color, Padding
 
-from subiquity.models.filesystem import _humanize_size
+from subiquity.models.filesystem import humanize_size
 
 log = logging.getLogger('subiquity.ui.bcache')
 
@@ -103,7 +103,7 @@ class BcacheView(BaseView):
             else:
                 bcachedev = device
 
-            disk_sz = _humanize_size(bcachedev.size)
+            disk_sz = humanize_size(bcachedev.size)
             disk_string = "{}     {},     {}".format(dname,
                                                      disk_sz,
                                                      device.model)
@@ -114,12 +114,9 @@ class BcacheView(BaseView):
 
     def _build_buttons(self):
         log.debug('bcache: _build_buttons')
-        cancel = cancel_btn(on_press=self.cancel)
-        done = done_btn(on_press=self.done)
-
         buttons = [
-            Color.button(done),
-            Color.button(cancel)
+            done_btn(on_press=self.done),
+            cancel_btn(on_press=self.cancel),
         ]
         return Pile(buttons)
 
