@@ -67,7 +67,7 @@ from subiquity.models.filesystem import (
     humanize_size,
     )
 
-from .delete import ConfirmDeleteStretchy
+from .delete import ConfirmDeleteStretchy, ConfirmReformatStretchy
 from .disk_info import DiskInfoStretchy
 from .helpers import summarize_device
 from .lvm import VolGroupStretchy
@@ -129,11 +129,20 @@ class MountInfo:
 
     @property
     def fstype(self):
-        return self.mount.device.fstype
+        fstype = self.mount.device.fstype
+        if self.mount.device.preserve:
+            fstype = "existing" + " " + fstype
+        else:
+            fstype = "new" + " " + fstype
+        return fstype
 
     @property
     def desc(self):
-        return self.mount.device.volume.desc()
+        annotations = self.mount.device.volume.annotations
+        desc = self.mount.device.volume.desc()
+        if annotations:
+            desc = annotations[0] + " " + desc
+        return desc
 
     def startswith(self, other):
         i = 0
@@ -289,6 +298,7 @@ class DeviceList(WidgetWrap):
         super().__init__(self.table)
 
     _disk_INFO = _stretchy_shower(DiskInfoStretchy)
+    _disk_REFORMAT = _stretchy_shower(ConfirmReformatStretchy)
     _disk_PARTITION = _stretchy_shower(PartitionStretchy)
     _disk_FORMAT = _stretchy_shower(FormatEntireStretchy)
 
@@ -353,7 +363,8 @@ class DeviceList(WidgetWrap):
             else:
                 meth_name = '_{}_{}'.format(device.type, action.name)
                 meth = getattr(self, meth_name)
-            if not whynot and action == DeviceAction.DELETE:
+            if not whynot and action in [DeviceAction.DELETE,
+                                         DeviceAction.REFORMAT]:
                 label = Color.danger_button(ActionMenuOpenButton(label))
             device_actions.append(Action(
                 label=label,
