@@ -25,23 +25,18 @@ class BaseController(ABC):
 
     signals = []
 
-    def __init__(self, common):
-        self.ui = common['ui']
-        self.signal = common['signal']
-        self.opts = common['opts']
-        self.loop = common['loop']
-        self.prober = common['prober']
-        self.controllers = common['controllers']
-        self.pool = common['pool']
-        self.base_model = common['base_model']
-        self.all_answers = common['answers']
-        self.input_filter = common['input_filter']
-        self.scale_factor = common['scale_factor']
-        self.run_in_bg = common['run_in_bg']
-        self.updated = common['updated']
-        self.application = common['application']
-        if 'snapd_connection' in common:
-            self.snapd_connection = common['snapd_connection']
+    @classmethod
+    def _controller_name(cls):
+        return cls.__name__[:-len("Controller")]
+
+    def __init__(self, app):
+        self.ui = app.ui
+        self.signal = app.signal
+        self.opts = app.opts
+        self.loop = app.loop
+        self.run_in_bg = app.run_in_bg
+        self.app = app
+        self.answers = app.answers.get(self._controller_name(), {})
 
     def register_signals(self):
         """Defines signals associated with controller from model."""
@@ -51,15 +46,41 @@ class BaseController(ABC):
         self.signal.connect_signals(signals)
 
     def start(self):
+        """Called just before the main loop is started.
+
+        At the time this is called, all controllers and models and so on
+        have been created. This is when the controller should start
+        interacting with the outside world, e.g. probing for network
+        devices or start making connections to the snap store.
+        """
         pass
 
     @abstractmethod
     def cancel(self):
         pass
 
+    @property
+    def showing(self):
+        cur_controller = self.app.cur_controller
+        while isinstance(cur_controller, RepeatedController):
+            cur_controller = cur_controller.orig
+        return cur_controller is self
+
     @abstractmethod
-    def default(self):
-        pass
+    def start_ui(self):
+        """Start running this controller's UI.
+
+        This method should call self.ui.set_body.
+        """
+
+    def end_ui(self):
+        """Stop running this controller's UI.
+
+        This method doesn't actually need to remove this controller's UI
+        as the next one is about to replace it, it's more of a hook to
+        stop any background tasks that can be stopped when the UI is not
+        running.
+        """
 
     def serialize(self):
         return None
@@ -99,7 +120,7 @@ class BaseController(ABC):
 
     def _run_iterator(self, it, delay=None):
         if delay is None:
-            delay = 0.2/self.scale_factor
+            delay = 0.2/self.app.scale_factor
         try:
             next(it)
         except StopIteration:
@@ -118,8 +139,8 @@ class RepeatedController(BaseController):
     def register_signals(self):
         pass
 
-    def default(self):
-        self.orig.default(self.index)
+    def start_ui(self):
+        self.orig.start_ui(self.index)
 
     def cancel(self):
         self.orig.cancel()
