@@ -183,10 +183,9 @@ class NetworkController(BaseController):
 
     root = "/"
 
-    def __init__(self, common):
-        super().__init__(common)
-        self.model = self.base_model.network
-        self.answers = self.all_answers.get("Network", {})
+    def __init__(self, app):
+        super().__init__(app)
+        self.model = app.base_model.network
         self.view = None
         self.view_shown = False
         self.dhcp_check_handle = None
@@ -213,7 +212,7 @@ class NetworkController(BaseController):
     def start(self):
         self._observer_handles = []
         self.observer, self._observer_fds = (
-            self.prober.probe_network(self.network_event_receiver))
+            self.app.prober.probe_network(self.network_event_receiver))
         self.start_watching()
 
     def stop_watching(self):
@@ -236,7 +235,7 @@ class NetworkController(BaseController):
             self.loop.set_alarm_in(0.1, lambda loop, ud: self.start_watching())
             return
         self.observer.data_ready(fd)
-        v = self.ui.frame.body
+        v = self.ui.body
         if hasattr(v, 'refresh_model_inputs'):
             v.refresh_model_inputs()
 
@@ -245,13 +244,11 @@ class NetworkController(BaseController):
 
     def done(self):
         log.debug("NetworkController.done next-screen")
-        self.view = None
         self.model.has_network = bool(
             self.network_event_receiver.default_routes)
         self.signal.emit_signal('next-screen')
 
     def cancel(self):
-        self.view = None
         self.signal.emit_signal('prev-screen')
 
     def _action_get(self, id):
@@ -281,11 +278,11 @@ class NetworkController(BaseController):
         if 'obj' in action:
             obj = self._action_get(action['obj'])
             meth = getattr(
-                self.ui.frame.body,
+                self.ui.body,
                 "_action_{}".format(action['action']))
             meth(obj)
             yield
-            body = self.ui.frame.body._w
+            body = self.ui.body._w
             if not isinstance(body, StretchyOverlay):
                 return
             for k, v in action.items():
@@ -302,15 +299,15 @@ class NetworkController(BaseController):
                     v,
                     action.get(submit_key, True))
         elif action['action'] == 'create-bond':
-            self.ui.frame.body._create_bond()
+            self.ui.body._create_bond()
             yield
-            body = self.ui.frame.body._w
+            body = self.ui.body._w
             yield from self._enter_form_data(
                 body.stretchy.form,
                 action['data'],
                 action.get("submit", True))
         elif action['action'] == 'done':
-            self.ui.frame.body.done()
+            self.ui.body.done()
         else:
             raise Exception("could not process action {}".format(action))
 
@@ -340,7 +337,7 @@ class NetworkController(BaseController):
                 dev.set_dhcp_state(v, "TIMEDOUT")
                 self.network_event_receiver.update_link(dev.ifindex)
 
-    def default(self):
+    def start_ui(self):
         if not self.view_shown:
             self.update_initial_configs()
         self.view = NetworkView(self.model, self)
@@ -350,6 +347,9 @@ class NetworkController(BaseController):
         self.network_event_receiver.view = self.view
         self.ui.set_body(self.view)
 
+    def end_ui(self):
+        self.view = self.network_event_receiver.view = None
+
     @property
     def netplan_path(self):
         if self.opts.project == "subiquity":
@@ -402,7 +402,7 @@ class NetworkController(BaseController):
         self.model.parse_netplan_configs(self.root)
 
         if self.opts.dry_run:
-            delay = 0.1/self.scale_factor
+            delay = 0.1/self.app.scale_factor
             tasks = [
                 ('one', BackgroundProcess(['sleep', str(delay)])),
                 ('two', PythonSleep(delay)),
