@@ -19,8 +19,8 @@ import json
 import logging
 import os
 import struct
+import subprocess
 import sys
-import termios
 import tty
 
 import urwid
@@ -29,6 +29,7 @@ import yaml
 from subiquitycore.controller import RepeatedController
 from subiquitycore.signals import Signal
 from subiquitycore.prober import Prober, ProberException
+from subiquitycore.ui.frame import SubiquityCoreUI
 
 log = logging.getLogger('subiquitycore.core')
 
@@ -156,25 +157,13 @@ class KeyCodesFilter:
     def enter_keycodes_mode(self):
         log.debug("enter_keycodes_mode")
         self.filtering = True
-        # Read the old keyboard mode (it will proably always be
-        # K_UNICODE but well).
+        # Read the old keyboard mode (it will proably always be K_UNICODE but
+        # well).
         o = bytearray(4)
         fcntl.ioctl(self._fd, KDGKBMODE, o)
         self._old_mode = struct.unpack('i', o)[0]
-        # Make some changes to the terminal settings.
-        # If you don't do this, sometimes writes to the terminal hang (and no,
-        # I don't know exactly why).
-        self._old_settings = termios.tcgetattr(self._fd)
-        new_settings = termios.tcgetattr(self._fd)
-        new_settings[tty.IFLAG] = 0
-        new_settings[tty.LFLAG] = new_settings[tty.LFLAG] & ~(termios.ECHO |
-                                                              termios.ICANON |
-                                                              termios.ISIG)
-        new_settings[tty.CC][termios.VMIN] = 0
-        new_settings[tty.CC][termios.VTIME] = 0
-        termios.tcsetattr(self._fd, termios.TCSAFLUSH, new_settings)
-        # Finally, set the keyboard mode to K_MEDIUMRAW, which causes
-        # the keyboard driver in the kernel to pass us keycodes.
+        # Set the keyboard mode to K_MEDIUMRAW, which causes the keyboard
+        # driver in the kernel to pass us keycodes.
         log.debug("old mode was %s, setting mode to %s",
                   self._old_mode, K_MEDIUMRAW)
         fcntl.ioctl(self._fd, KDSKBMODE, K_MEDIUMRAW)
@@ -184,7 +173,6 @@ class KeyCodesFilter:
         self.filtering = False
         log.debug("setting mode back to %s", self._old_mode)
         fcntl.ioctl(self._fd, KDSKBMODE, self._old_mode)
-        termios.tcsetattr(self._fd, termios.TCSANOW, self._old_settings)
 
     def filter(self, keys, codes):
         # Luckily urwid passes us the raw results from read() we can
@@ -244,7 +232,9 @@ class Application:
     # controllers in order, calling the default method on the controller
     # instance.
 
-    def __init__(self, ui, opts):
+    make_ui = SubiquityCoreUI
+
+    def __init__(self, opts):
         try:
             prober = Prober(opts)
         except ProberException as e:
@@ -252,6 +242,8 @@ class Application:
             log.exception(err)
             raise ApplicationError(err)
 
+        self.ui = self.make_ui()
+        self.opts = opts
         opts.project = self.project
 
         self.root = '/'
@@ -260,44 +252,42 @@ class Application:
         self.state_dir = os.path.join(self.root, 'run', self.project)
         os.makedirs(os.path.join(self.state_dir, 'states'), exist_ok=True)
 
-        answers = {}
+        self.answers = {}
         if opts.answers is not None:
-            answers = yaml.safe_load(opts.answers.read())
-            log.debug("Loaded answers %s", answers)
+            self.answers = yaml.safe_load(opts.answers.read())
+            log.debug("Loaded answers %s", self.answers)
             if not opts.dry_run:
                 open('/run/casper-no-prompt', 'w').close()
 
         if is_linux_tty():
             log.debug("is_linux_tty")
-            input_filter = KeyCodesFilter()
+            self.input_filter = KeyCodesFilter()
         else:
-            input_filter = DummyKeycodesFilter()
+            self.input_filter = DummyKeycodesFilter()
 
-        scale = float(os.environ.get('SUBIQUITY_REPLAY_TIMESCALE', "1"))
-        updated = os.path.exists(os.path.join(self.state_dir, 'updating'))
-        self.common = {
-            "application": self,
-            "updated": updated,
-            "ui": ui,
-            "opts": opts,
-            "signal": Signal(),
-            "prober": prober,
-            "loop": None,
-            "pool": futures.ThreadPoolExecutor(10),
-            "answers": answers,
-            "input_filter": input_filter,
-            "scale_factor": scale,
-            "run_in_bg": self.run_in_bg,
-        }
+        self.scale_factor = float(
+            os.environ.get('SUBIQUITY_REPLAY_TIMESCALE', "1"))
+        self.updated = os.path.exists(os.path.join(self.state_dir, 'updating'))
+        self.signal = Signal()
+        self.prober = prober
+        self.loop = None
+        self.pool = futures.ThreadPoolExecutor(10)
         if opts.screens:
             self.controllers = [c for c in self.controllers
                                 if c in opts.screens]
         else:
             self.controllers = self.controllers[:]
-        ui.progress_completion = len(self.controllers)
-        self.common['controllers'] = dict.fromkeys(self.controllers)
+        self.ui.progress_completion = len(self.controllers)
+        self.controller_instances = dict.fromkeys(self.controllers)
         self.controller_index = -1
 
+    @property
+    def cur_controller(self):
+        if self.controller_index < 0:
+            return None
+        controller_name = self.controllers[self.controller_index]
+        return self.controller_instances[controller_name]
+
     def run_in_bg(self, func, callback):
         """Run func() in a thread and call callback on UI thread.
 
@@ -305,55 +295,79 @@ class Application:
         the result of func(). The result of callback is discarded. An
         exception will crash the process so be careful!
         """
-        fut = self.common['pool'].submit(func)
+        fut = self.pool.submit(func)
 
         def in_main_thread(ignored):
+            self.loop.remove_watch_pipe(pipe)
+            os.close(pipe)
             callback(fut)
 
-        pipe = self.common['loop'].watch_pipe(in_main_thread)
+        pipe = self.loop.watch_pipe(in_main_thread)
 
         def in_random_thread(ignored):
             os.write(pipe, b'x')
         fut.add_done_callback(in_random_thread)
 
+    def run_command_in_foreground(self, cmd, **kw):
+        screen = self.loop.screen
+
+        def run():
+            subprocess.run(cmd, **kw)
+
+        def restore(fut):
+            screen.start()
+            # Calling screen.start() sends the INPUT_DESCRIPTORS_CHANGED
+            # signal. This calls _reset_input_descriptors() which calls
+            # unhook_event_loop / hook_event_loop on the screen. But this all
+            # happens before _started is set on the screen, so hook_event_loop
+            # does not actually do anything -- and we end up not listening to
+            # stdin, obviously a defective situation for a console
+            # application. So send it again now the screen is started...
+            urwid.emit_signal(
+                screen, urwid.display_common.INPUT_DESCRIPTORS_CHANGED)
+            tty.setraw(0)
+
+        screen.stop()
+        self.run_in_bg(run, restore)
+
     def _connect_base_signals(self):
         """ Connect signals used in the core controller
         """
         signals = []
 
         signals.append(('quit', self.exit))
-        if self.common['opts'].dry_run:
+        if self.opts.dry_run:
             signals.append(('control-x-quit', self.exit))
         signals.append(('refresh', self.redraw_screen))
         signals.append(('next-screen', self.next_screen))
         signals.append(('prev-screen', self.prev_screen))
-        self.common['signal'].connect_signals(signals)
+        self.signal.connect_signals(signals)
 
         # Registers signals from each controller
-        for controller, controller_class in self.common['controllers'].items():
+        for controller_class in self.controller_instances.values():
             controller_class.register_signals()
-        log.debug(self.common['signal'])
+        log.debug(self.signal)
 
     def save_state(self):
-        if self.controller_index < 0:
+        cur_controller = self.cur_controller
+        if cur_controller is None:
             return
-        cur_controller_name = self.controllers[self.controller_index]
-        cur_controller = self.common['controllers'][cur_controller_name]
         state_path = os.path.join(
-            self.state_dir, 'states', cur_controller_name)
+            self.state_dir, 'states', cur_controller._controller_name())
         with open(state_path, 'w') as fp:
             json.dump(cur_controller.serialize(), fp)
 
     def select_screen(self, index):
+        if self.cur_controller is not None:
+            self.cur_controller.end_ui()
         self.controller_index = index
-        self.common['ui'].progress_current = index
-        controller_name = self.controllers[self.controller_index]
-        log.debug("moving to screen %s", controller_name)
-        controller = self.common['controllers'][controller_name]
-        controller.default()
+        self.ui.progress_current = index
+        log.debug(
+            "moving to screen %s", self.cur_controller._controller_name())
+        self.cur_controller.start_ui()
         state_path = os.path.join(self.state_dir, 'last-screen')
         with open(state_path, 'w') as fp:
-            fp.write(controller_name)
+            fp.write(self.cur_controller._controller_name())
 
     def next_screen(self, *args):
         self.save_state()
@@ -385,9 +399,9 @@ class Application:
 
 # EventLoop -------------------------------------------------------------------
     def redraw_screen(self):
-        if self.common['loop'] is not None:
+        if self.loop is not None:
             try:
-                self.common['loop'].draw_screen()
+                self.loop.draw_screen()
             except AssertionError as e:
                 log.critical("Redraw screen error: {}".format(e))
 
@@ -408,7 +422,6 @@ class Application:
         #    wait, above to wait for the button to appear in case it
         #    takes a while.
         from subiquitycore.testing import view_helpers
-        loop = self.common['loop']
 
         class ScriptState:
             def __init__(self):
@@ -426,11 +439,10 @@ class Application:
                 return
             ss.scripts = ss.scripts[1:]
             if ss.scripts:
-                loop.set_alarm_in(0.01, _run_script)
+                self.loop.set_alarm_in(0.01, _run_script)
 
         def c(pat):
-            but = view_helpers.find_button_matching(self.common['ui'],
-                                                    '.*' + pat + '.*')
+            but = view_helpers.find_button_matching(self.ui, '.*' + pat + '.*')
             if not but:
                 ss.wait_count += 1
                 if ss.wait_count > 10:
@@ -452,73 +464,85 @@ class Application:
                     ss.scripts = ss.scripts[1:]
                     if ss.scripts:
                         _run_script()
-            loop.set_alarm_in(delay, next)
+            self.loop.set_alarm_in(delay, next)
 
         ss.ns['c'] = c
         ss.ns['wait'] = wait
-        ss.ns['ui'] = self.common['ui']
+        ss.ns['ui'] = self.ui
 
-        self.common['loop'].set_alarm_in(0.06, _run_script)
+        self.loop.set_alarm_in(0.06, _run_script)
+
+    def unhandled_input(self, key):
+        if key == 'ctrl x':
+            self.signal.emit_signal('control-x-quit')
+
+    def load_controllers(self):
+        controllers_mod = __import__(
+            '{}.controllers'.format(self.project), None, None, [''])
+        for i, k in enumerate(self.controllers):
+            if self.controller_instances[k] is None:
+                log.debug("Importing controller: {}".format(k))
+                klass = getattr(controllers_mod, k+"Controller")
+                self.controller_instances[k] = klass(self)
+            else:
+                count = 1
+                for k2 in self.controllers[:i]:
+                    if k2 == k or k2.startswith(k + '-'):
+                        count += 1
+                orig = self.controller_instances[k]
+                k += '-' + str(count)
+                self.controllers[i] = k
+                self.controller_instances[k] = RepeatedController(
+                    orig, count)
+        log.debug("*** %s", self.controller_instances)
+
+    def load_serialized_state(self):
+        for k in self.controllers:
+            state_path = os.path.join(self.state_dir, 'states', k)
+            if not os.path.exists(state_path):
+                continue
+            with open(state_path) as fp:
+                self.controller_instances[k].deserialize(
+                    json.load(fp))
+
+        last_screen = None
+        state_path = os.path.join(self.state_dir, 'last-screen')
+        if os.path.exists(state_path):
+            with open(state_path) as fp:
+                last_screen = fp.read().strip()
+
+        if last_screen in self.controllers:
+            return self.controllers.index(last_screen)
+        else:
+            return 0
 
     def run(self):
-        if not hasattr(self, 'loop'):
-            if (self.common['opts'].run_on_serial and
-                    os.ttyname(0) != "/dev/ttysclp0"):
-                palette = self.STYLES_MONO
-                screen = urwid.raw_display.Screen()
-            else:
-                screen, palette = setup_screen(self.COLORS, self.STYLES)
+        if (self.opts.run_on_serial and
+                os.ttyname(0) != "/dev/ttysclp0"):
+            palette = self.STYLES_MONO
+            screen = urwid.raw_display.Screen()
+        else:
+            screen, palette = setup_screen(self.COLORS, self.STYLES)
 
-            self.common['loop'] = urwid.MainLoop(
-                self.common['ui'], palette=palette, screen=screen,
-                handle_mouse=False, pop_ups=True,
-                input_filter=self.common['input_filter'].filter)
+        self.loop = urwid.MainLoop(
+            self.ui, palette=palette, screen=screen,
+            handle_mouse=False, pop_ups=True,
+            input_filter=self.input_filter.filter,
+            unhandled_input=self.unhandled_input)
 
-            log.debug("Running event loop: {}".format(
-                self.common['loop'].event_loop))
-            self.common['base_model'] = self.make_model(self.common)
+        log.debug("Running event loop: {}".format(
+            self.loop.event_loop))
+        self.base_model = self.make_model()
         try:
-            if self.common['opts'].scripts:
-                self.run_scripts(self.common['opts'].scripts)
-            controllers_mod = __import__('%s.controllers' % self.project,
-                                         None, None, [''])
-            for i, k in enumerate(self.controllers):
-                if self.common['controllers'][k] is None:
-                    log.debug("Importing controller: {}".format(k))
-                    klass = getattr(controllers_mod, k+"Controller")
-                    self.common['controllers'][k] = klass(self.common)
-                else:
-                    count = 1
-                    for k2 in self.controllers[:i]:
-                        if k2 == k or k2.startswith(k + '-'):
-                            count += 1
-                    orig = self.common['controllers'][k]
-                    k += '-' + str(count)
-                    self.controllers[i] = k
-                    self.common['controllers'][k] = RepeatedController(
-                        orig, count)
-            log.debug("*** %s", self.common['controllers'])
+            if self.opts.scripts:
+                self.run_scripts(self.opts.scripts)
+
+            self.load_controllers()
 
             initial_controller_index = 0
 
-            if self.common['updated']:
-                for k in self.controllers:
-                    state_path = os.path.join(self.state_dir, 'states', k)
-                    if not os.path.exists(state_path):
-                        continue
-                    with open(state_path) as fp:
-                        self.common['controllers'][k].deserialize(
-                            json.load(fp))
-
-                last_screen = None
-                state_path = os.path.join(self.state_dir, 'last-screen')
-                if os.path.exists(state_path):
-                    with open(state_path) as fp:
-                        last_screen = fp.read().strip()
-
-                if last_screen in self.controllers:
-                    initial_controller_index = self.controllers.index(
-                        last_screen)
+            if self.updated:
+                initial_controller_index = self.load_serialized_state()
 
             def select_initial_screen(loop, index):
                 try:
@@ -526,14 +550,16 @@ class Application:
                 except Skip:
                     self.next_screen()
 
-            self.common['loop'].set_alarm_in(
+            self.loop.set_alarm_in(
+                0.00, lambda loop, ud: tty.setraw(0))
+            self.loop.set_alarm_in(
                 0.05, select_initial_screen, initial_controller_index)
             self._connect_base_signals()
 
             for k in self.controllers:
-                self.common['controllers'][k].start()
+                self.controller_instances[k].start()
 
-            self.common['loop'].run()
+            self.loop.run()
         except Exception:
             log.exception("Exception in controller.run():")
             raise
