@@ -14,12 +14,11 @@
 # along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
 import crypt
-import errno
 import logging
 import os
 import random
 import yaml
-from subprocess import Popen, PIPE
+import subprocess
 
 log = logging.getLogger("subiquitycore.utils")
 SYS_CLASS_NET = "/sys/class/net/"
@@ -68,10 +67,19 @@ def environment_check(check):
         for ftype, items in checks[check_type].items():
             for i in items:
                 if not os.path.exists(i):
-                    log.error('FAIL: {} is not found on the'
-                              ' filesystem'.format(i))
-                    env_ok = False
-                    continue
+                    if 'SNAP' in os.environ:
+                        log.warn("Adjusting path for snaps: %s",
+                                 os.environ.get('SNAP'))
+                        i = os.environ.get('SNAP') + i
+                        if not os.path.exists(i):
+                            env_ok = False
+                    else:
+                        env_ok = False
+
+                    if not env_ok:
+                        log.error('FAIL: {} is not found on the'
+                                  ' filesystem'.format(i))
+                        continue
                 if check_map[ftype](i) is False:
                     log.error('FAIL: {} is NOT of type: {}'.format(i, ftype))
                     env_ok = False
@@ -84,63 +92,55 @@ def environment_check(check):
     return env_ok
 
 
-def run_command_start(command, timeout=None, shell=False):
-    log.debug('run_command called: {}'.format(command))
-    cmd_env = os.environ.copy()
-    # set consistent locale
-    cmd_env['LC_ALL'] = 'C'
-    if timeout:
-        command = "timeout %ds %s" % (timeout, command)
-
-    try:
-        log.debug('trying Popen...')
-        # dummy stdin fd as per:
-        # http://stackoverflow.com/ +
-        # questions/27022810/urwid-watch-file-blocks-keypress
-        r, w = os.pipe()
-        p = Popen(command, shell=shell,
-                  stdin=r, stdout=PIPE, stderr=PIPE,
-                  bufsize=-1, env=cmd_env, close_fds=True)
-        os.close(w)
-    except OSError as e:
-        if e.errno == errno.ENOENT:
-            log.debug('error!')
-            return dict(ret=127, output="", err="")
-        else:
-            log.debug('error raise!')
-            raise e
-    return p
+def _clean_env(env):
+    if env is None:
+        env = os.environ.copy()
+    else:
+        env = env.copy()
+    # Do we always want to do this?
+    env['LC_ALL'] = 'C'
+    # Maaaybe want to remove SNAP here too.
+    return env
 
 
-def run_command_summarize(p, stdout, stderr):
-    if p.returncode == 126 or p.returncode == 127:
-        stdout = bytes()
-    if not stderr:
-        stderr = bytes()
-    rv = dict(status=p.returncode,
-              output=stdout.decode('utf-8'),
-              err=stderr.decode('utf-8'))
-    log.debug('run_command returning: {}'.format(rv))
-    return rv
+def run_command(cmd, *, input=None, stdout=subprocess.PIPE,
+                stderr=subprocess.PIPE, encoding='utf-8', errors='replace',
+                env=None, **kw):
+    """A wrapper around subprocess.run with logging and different defaults.
 
-
-def run_command(command, timeout=None, shell=False):
-    """ Execute command through system shell
-    :param command: command to run
-    :param timeout: (optional) use 'timeout' to limit time. default 300
-    :type command: str
-    :returns: {status: returncode, output: stdout, err: stderr}
-    :rtype: dict
-    .. code::
-        # Get output of juju status
-        cmd_dict = utils.get_command_output('juju status')
+    We never ever want a subprocess to inherit our file descriptors!
     """
-    p = run_command_start(command, timeout, shell)
-    log.debug('calling communicate()')
-    if isinstance(p, dict): # only in error cases
-        return p
-    stdout, stderr = p.communicate()
-    return run_command_summarize(p, stdout, stderr)
+    if input is None:
+        kw['stdin'] = subprocess.DEVNULL
+    else:
+        input = input.encode(encoding)
+    log.debug("run_command called: %s", cmd)
+    try:
+        cp = subprocess.run(cmd, input=input, stdout=stdout, stderr=stderr,
+                            env=_clean_env(env), **kw)
+        if encoding:
+            if isinstance(cp.stdout, bytes):
+                cp.stdout = cp.stdout.decode(encoding)
+            if isinstance(cp.stderr, bytes):
+                cp.stderr = cp.stderr.decode(encoding)
+    except subprocess.CalledProcessError as e:
+        log.debug("run_command %s", str(e))
+        raise
+    else:
+        log.debug("run_command %s exited with code %s", cp.args, cp.returncode)
+        return cp
+
+
+def start_command(cmd, *, stdin=subprocess.DEVNULL, stdout=subprocess.PIPE,
+                  stderr=subprocess.PIPE, encoding='utf-8', errors='replace',
+                  env=None, **kw):
+    """A wrapper around subprocess.Popen with logging and different defaults.
+
+    We never ever want a subprocess to inherit our file descriptors!
+    """
+    log.debug('start_command called: %s', cmd)
+    return subprocess.Popen(cmd, stdin=stdin, stdout=stdout, stderr=stderr,
+                            env=_clean_env(env), **kw)
 
 
 # FIXME: replace with passlib and update package deps
@@ -162,13 +162,18 @@ def crypt_password(passwd, algo='SHA-512'):
 def disable_console_conf():
     """ Stop console-conf service; which also restores getty service """
     log.info('disabling console-conf service')
-    run_command(["systemctl", "stop", "--no-block", "console-conf@*.service", "serial-console-conf@*.service"])
+    run_command(["systemctl", "stop", "--no-block", "console-conf@*.service",
+                 "serial-console-conf@*.service"])
     return
 
+
 def disable_subiquity():
     """ Stop subiquity service; which also restores getty service """
     log.info('disabling subiquity service')
     run_command(["mkdir", "-p", "/run/subiquity"])
     run_command(["touch", "/run/subiquity/complete"])
-    run_command(["systemctl", "stop", "--no-block", "subiquity*.service"])
+    run_command(["systemctl", "start", "--no-block", "getty@tty1.service"])
+    run_command(["systemctl", "stop", "--no-block",
+                 "snap.subiquity.subiquity-service.service",
+                 "serial-subiquity@*.service"])
     return
